import * as tslib_1 from "tslib";
import { EventHandler } from 'phovea_core/src/event';
import { LayoutContainerEvents } from '../interfaces';
import { dragAble, uniqueId } from 'phovea_core/src';
export function withChanged(event) {
    return "" + event + EventHandler.MULTI_EVENT_SEPARATOR + LayoutContainerEvents.EVENT_LAYOUT_CHANGED;
}
var ALayoutContainer = /** @class */ (function (_super) {
    tslib_1.__extends(ALayoutContainer, _super);
    function ALayoutContainer(document, options) {
        var _this = _super.call(this) || this;
        _this.parent = null;
        _this.id = uniqueId(ALayoutContainer.MIME_TYPE);
        _this.keyDownListener = function (evt) {
            if (evt.keyCode === 27) { // Escape
                _this.toggleMaximizedView();
            }
        };
        _this.isMaximized = false;
        console.assert(document != null);
        _this.options = Object.assign(_this.defaultOptions(), options);
        if (_this.options.fixed) {
            _this.options.fixedLayout = true;
        }
        _this.header = document.createElement('header');
        _this.header.innerHTML = "\n        <button type=\"button\" class=\"close" + (_this.options.fixed ? ' hidden' : '') + "\" aria-label=\"Close\"><span>\u00D7</span></button>\n        <span>" + _this.name + "</span>";
        //remove
        _this.header.firstElementChild.addEventListener('click', function (evt) {
            evt.preventDefault();
            evt.stopPropagation();
            _this.destroy();
        });
        //drag
        if (!_this.options.fixedLayout) {
            dragAble(_this.header, function () {
                return {
                    effectAllowed: 'move',
                    data: (_a = {
                            'text/plain': _this.name
                        },
                        _a[ALayoutContainer.MIME_TYPE] = String(_this.id),
                        _a)
                };
                var _a;
            }, true);
        }
        return _this;
    }
    Object.defineProperty(ALayoutContainer.prototype, "parents", {
        get: function () {
            var r = [];
            var p = this.parent;
            while (p !== null) {
                r.push(p);
                p = p.parent;
            }
            return r;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ALayoutContainer.prototype, "hideAbleHeader", {
        get: function () {
            return false;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(ALayoutContainer.prototype, "autoWrapOnDrop", {
        get: function () {
            return this.options.autoWrap;
        },
        enumerable: true,
        configurable: true
    });
    ALayoutContainer.prototype.defaultOptions = function () {
        return {
            name: 'View',
            fixed: false,
            hideAbleHeader: false,
            autoWrap: false,
            fixedLayout: false
        };
    };
    ALayoutContainer.prototype.destroy = function () {
        this.fire(withChanged(LayoutContainerEvents.EVENT_DESTROYED), this);
    };
    Object.defineProperty(ALayoutContainer.prototype, "name", {
        get: function () {
            return this.options.name;
        },
        set: function (name) {
            if (this.options.name === name) {
                return;
            }
            this.fire(withChanged(LayoutContainerEvents.EVENT_NAME_CHANGED), this.options.name, this.options.name = name);
            this.updateName(name);
        },
        enumerable: true,
        configurable: true
    });
    ALayoutContainer.prototype.updateName = function (name) {
        this.header.children[1].textContent = name;
    };
    ALayoutContainer.prototype.persist = function () {
        return {
            type: '',
            name: this.name,
            fixed: this.options.fixed,
            autoWrap: this.options.autoWrap,
            fixedLayout: this.options.fixedLayout
        };
    };
    ALayoutContainer.restoreOptions = function (dump) {
        return {
            name: dump.name,
            fixed: dump.fixed,
            autoWrap: dump.autoWrap === true,
            fixedLayout: dump.fixedLayout === true
        };
    };
    ALayoutContainer.deriveOptions = function (node) {
        return {
            name: node.dataset.name || 'View',
            fixed: node.dataset.fixed !== undefined,
            autoWrap: node.dataset.autoWrap !== undefined,
            fixedLayout: node.dataset.fixedLayout !== undefined
        };
    };
    ALayoutContainer.prototype.find = function (id) {
        return (typeof id === 'number' && this.id === id) || (typeof id === 'function' && id(this)) ? this : null;
    };
    ALayoutContainer.prototype.findAll = function (predicate) {
        return predicate(this) ? [this] : [];
    };
    ALayoutContainer.prototype.closest = function (id) {
        if (!this.parent) {
            return null;
        }
        var parent = this.parent;
        if ((typeof id === 'number' && parent.id === id) || (typeof id === 'function' && id(parent))) {
            return parent;
        }
        return parent.closest(id);
    };
    ALayoutContainer.prototype.toggleMaximizedView = function () {
        var sizeToggle = this.header.querySelector('.size-toggle');
        var sizeToggleIcon = sizeToggle.querySelector('i');
        var closeButton = this.header.querySelector('.close');
        this.isMaximized = !this.isMaximized;
        if (this.isMaximized) {
            closeButton.classList.add('hidden');
            sizeToggle.title = 'Restore default size';
            sizeToggleIcon.classList.remove('fa-expand');
            sizeToggleIcon.classList.add('fa-compress');
            this.header.ownerDocument.addEventListener('keydown', this.keyDownListener);
            this.fire(LayoutContainerEvents.EVENT_MAXIMIZE, this);
        }
        else {
            if (!this.options.fixedLayout) {
                closeButton.classList.remove('hidden');
            }
            sizeToggle.title = 'Expand view';
            sizeToggleIcon.classList.add('fa-expand');
            sizeToggleIcon.classList.remove('fa-compress');
            this.header.ownerDocument.removeEventListener('keydown', this.keyDownListener);
            this.fire(LayoutContainerEvents.EVENT_RESTORE_SIZE, this);
        }
        this.updateTitle();
    };
    ALayoutContainer.prototype.updateTitle = function () {
        this.header.title = "Double click to " + (this.isMaximized ? 'restore default size' : 'expand view');
    };
    ALayoutContainer.MIME_TYPE = 'text/x-phovea-layout-container';
    return ALayoutContainer;
}(EventHandler));
export { ALayoutContainer };
export default ALayoutContainer;
//# sourceMappingURL=ALayoutContainer.js.map