import * as tslib_1 from "tslib";
import { LayoutContainerEvents } from '../interfaces';
import { ALayoutContainer, withChanged } from './ALayoutContainer';
var AParentLayoutContainer = /** @class */ (function (_super) {
    tslib_1.__extends(AParentLayoutContainer, _super);
    function AParentLayoutContainer(document, options) {
        var _this = _super.call(this, document, options) || this;
        _this._children = [];
        _this._visible = false;
        _this.node = document.createElement('main');
        _this.node.classList.add('phovea-layout');
        return _this;
    }
    AParentLayoutContainer.prototype.canDrop = function (area) {
        return false;
    };
    Object.defineProperty(AParentLayoutContainer.prototype, "rootParent", {
        get: function () {
            var p = this;
            while (p.parent !== null) {
                p = p.parent;
            }
            return p;
        },
        enumerable: true,
        configurable: true
    });
    AParentLayoutContainer.prototype.forEach = function (callback) {
        this._children.forEach(callback);
    };
    Object.defineProperty(AParentLayoutContainer.prototype, "children", {
        get: function () {
            return this._children.slice();
        },
        enumerable: true,
        configurable: true
    });
    AParentLayoutContainer.prototype[Symbol.iterator] = function () {
        return this._children[Symbol.iterator]();
    };
    Object.defineProperty(AParentLayoutContainer.prototype, "length", {
        get: function () {
            return this._children.length;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(AParentLayoutContainer.prototype, "visible", {
        get: function () {
            return this._visible;
        },
        set: function (visible) {
            if (this._visible === visible) {
                return;
            }
            this.fire(withChanged(LayoutContainerEvents.EVENT_VISIBILITY_CHANGED), this._visible, this._visible = visible);
            this.visibilityChanged(visible);
        },
        enumerable: true,
        configurable: true
    });
    AParentLayoutContainer.prototype.visibilityChanged = function (visible) {
        this.forEach(function (c) { return c.visible = visible; });
    };
    AParentLayoutContainer.prototype.push = function (child, index) {
        if (index === void 0) { index = -1; }
        this.setupChild(child);
        if (index >= this._children.length || index < 0) {
            index = this._children.length;
            this._children.push(child);
        }
        else {
            this._children.splice(index, 0, child);
        }
        this.addedChild(child, index);
        return true;
    };
    AParentLayoutContainer.prototype.setupChild = function (child) {
        if (child.parent) {
            child.parent.remove(child);
        }
        child.parent = this;
    };
    AParentLayoutContainer.prototype.addedChild = function (child, index) {
        child.resized();
        this.propagate(child, LayoutContainerEvents.EVENT_LAYOUT_CHANGED, LayoutContainerEvents.EVENT_MAXIMIZE, LayoutContainerEvents.EVENT_RESTORE_SIZE);
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHILD_ADDED), child, index);
    };
    AParentLayoutContainer.prototype.replace = function (child, replacement) {
        var index = this._children.indexOf(child);
        console.assert(index >= 0);
        this.takeDownChild(child);
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHILD_REMOVED), child);
        this.setupChild(replacement);
        this._children.splice(index, 1, replacement);
        this.addedChild(replacement, index);
        return true;
    };
    AParentLayoutContainer.prototype.remove = function (child) {
        this.takeDownChild(child);
        this._children.splice(this._children.indexOf(child), 1);
        if (this.minChildCount > this.length && this.parent) {
            if (this.length > 1) {
                //remove and inline my children (just one since the remove will be called again
                this.parent.push(this._children[1]);
            }
            else if (this.length > 0) {
                this.parent.replace(this, this._children[0]);
            }
            else {
                this.parent.remove(this);
            }
        }
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHILD_REMOVED), child);
        return true;
    };
    AParentLayoutContainer.prototype.clear = function () {
        var _this = this;
        if (this.length === 0) {
            return;
        }
        this._children.forEach(function (old) { return _this.takeDownChild(old); });
        this._children.splice(0, this._children.length);
        if (this.minChildCount > this.length && this.parent) {
            this.parent.remove(this);
        }
        this.fire(withChanged(LayoutContainerEvents.EVENT_CHILD_REMOVED));
    };
    AParentLayoutContainer.prototype.takeDownChild = function (child) {
        this.stopPropagation(child, LayoutContainerEvents.EVENT_LAYOUT_CHANGED, LayoutContainerEvents.EVENT_MAXIMIZE, LayoutContainerEvents.EVENT_RESTORE_SIZE);
        child.visible = false;
        child.parent = null;
    };
    AParentLayoutContainer.prototype.resized = function () {
        this.forEach(function (d) { return d.resized(); });
    };
    AParentLayoutContainer.prototype.destroy = function () {
        _super.prototype.destroy.call(this);
        if (this.parent) {
            this.parent.remove(this);
        }
        this.forEach(function (d) { return d.destroy(); });
    };
    AParentLayoutContainer.prototype.persist = function () {
        return Object.assign(_super.prototype.persist.call(this), {
            children: this._children.map(function (d) { return d.persist(); })
        });
    };
    AParentLayoutContainer.prototype.find = function (id) {
        if (_super.prototype.find.call(this, id) != null) {
            return this;
        }
        for (var _i = 0, _a = this._children; _i < _a.length; _i++) {
            var child = _a[_i];
            var r = child.find(id);
            if (r != null) {
                return r;
            }
        }
        return null;
    };
    AParentLayoutContainer.prototype.findAll = function (predicate) {
        var base = _super.prototype.findAll.call(this, predicate);
        return base.concat.apply(base, this._children.map(function (d) { return d.findAll(predicate); }));
    };
    return AParentLayoutContainer;
}(ALayoutContainer));
export { AParentLayoutContainer };
export default AParentLayoutContainer;
//# sourceMappingURL=AParentLayoutContainer.js.map