import * as tslib_1 from "tslib";
import { AParentLayoutContainer } from './AParentLayoutContainer';
import { EOrientation } from './interfaces';
var ASequentialLayoutContainer = /** @class */ (function (_super) {
    tslib_1.__extends(ASequentialLayoutContainer, _super);
    function ASequentialLayoutContainer(document, options) {
        var _this = _super.call(this, document, options) || this;
        _this.node.dataset.layout = 'sequence';
        _this.node.dataset.orientation = _this.options.orientation === EOrientation.HORIZONTAL ? 'h' : 'v';
        return _this;
    }
    ASequentialLayoutContainer.prototype.canDrop = function (area) {
        return this.options.orientation === EOrientation.HORIZONTAL ? (area === 'left' || area === 'right' || area === 'horizontal-scroll') : (area === 'top' || area === 'bottom' || area === 'vertical-scroll');
    };
    ASequentialLayoutContainer.prototype.defaultOptions = function () {
        return Object.assign(_super.prototype.defaultOptions.call(this), {
            orientation: EOrientation.HORIZONTAL
        });
    };
    Object.defineProperty(ASequentialLayoutContainer.prototype, "hideAbleHeader", {
        get: function () {
            return this.options.fixedLayout;
        },
        enumerable: true,
        configurable: true
    });
    ASequentialLayoutContainer.prototype.getPadding = function () {
        return 0;
    };
    Object.defineProperty(ASequentialLayoutContainer.prototype, "minSize", {
        get: function () {
            console.assert(this.length >= 1);
            var padding = this.getPadding();
            switch (this.options.orientation) {
                case EOrientation.HORIZONTAL:
                    return this._children.reduce(function (a, c) {
                        var cmin = c.minSize;
                        return [a[0] + cmin[0], Math.max(a[1], cmin[1])];
                    }, [padding, 0]);
                case EOrientation.VERTICAL: {
                    return this._children.reduce(function (a, c) {
                        var cmin = c.minSize;
                        return [Math.max(a[0], cmin[0]), a[1] + cmin[1]];
                    }, [0, padding]);
                }
            }
        },
        enumerable: true,
        configurable: true
    });
    ASequentialLayoutContainer.prototype.persist = function () {
        return Object.assign(_super.prototype.persist.call(this), {
            type: 'sequence',
            orientation: EOrientation[this.options.orientation]
        });
    };
    return ASequentialLayoutContainer;
}(AParentLayoutContainer));
export { ASequentialLayoutContainer };
export default ASequentialLayoutContainer;
export function wrap(child) {
    var s = child.node.ownerDocument.createElement('section');
    if (!child.hideAbleHeader) {
        s.appendChild(child.header);
    }
    s.appendChild(child.node);
    return s;
}
//# sourceMappingURL=ASequentialLayoutContainer.js.map