import * as tslib_1 from "tslib";
import { EOrientation } from './interfaces';
import { ALayoutContainer } from './ALayoutContainer';
import { ASequentialLayoutContainer, wrap } from './ASequentialLayoutContainer';
var LineUpLayoutContainer = /** @class */ (function (_super) {
    tslib_1.__extends(LineUpLayoutContainer, _super);
    function LineUpLayoutContainer(document, options) {
        var children = [];
        for (var _i = 2; _i < arguments.length; _i++) {
            children[_i - 2] = arguments[_i];
        }
        var _this = _super.call(this, document, options) || this;
        _this.minChildCount = 1;
        _this.type = 'lineup';
        _this.node.dataset.layout = 'lineup';
        if (_this.options.stackLayout) {
            _this.node.dataset.mode = 'stacked';
        }
        children.forEach(function (d) { return _this.push(d); });
        return _this;
    }
    LineUpLayoutContainer.prototype.place = function (child, reference, area) {
        console.assert(area !== 'center');
        var index = this._children.indexOf(reference) + (area === 'right' || area === 'bottom' ? 1 : 0);
        return this.push(child, index);
    };
    LineUpLayoutContainer.prototype.addedChild = function (child, index) {
        _super.prototype.addedChild.call(this, child, index);
        if (index < 0 || index >= (this._children.length - 1)) {
            //+1 since we already chanded the children
            this.node.appendChild(wrap(child));
        }
        else {
            this.node.insertBefore(wrap(child), this.node.children[index]);
        }
        if (this.options.stackLayout) {
            if (this.options.orientation === EOrientation.HORIZONTAL) {
                child.node.parentElement.style.minWidth = child.minSize[0] > 0 ? child.minSize[0] + "px" : null; // set minSize if available or delete CSS property
            }
            else {
                child.node.parentElement.style.minHeight = child.minSize[1] > 0 ? child.minSize[1] + "px" : null;
            }
        }
        child.visible = this.visible;
    };
    LineUpLayoutContainer.prototype.takeDownChild = function (child) {
        this.node.removeChild(child.node.parentElement);
        this.node.dataset.mode = '';
        _super.prototype.takeDownChild.call(this, child);
    };
    LineUpLayoutContainer.prototype.persist = function () {
        return Object.assign(_super.prototype.persist.call(this), {
            type: 'lineup',
            stackLayout: this.options.stackLayout
        });
    };
    LineUpLayoutContainer.restore = function (dump, restore, doc) {
        var options = Object.assign(ALayoutContainer.restoreOptions(dump), {
            orientation: EOrientation[dump.orientation],
            stackLayout: dump.stackLayout
        });
        var r = new LineUpLayoutContainer(doc, options);
        dump.children.forEach(function (d) { return r.push(restore(d)); });
        return r;
    };
    LineUpLayoutContainer.derive = function (node, derive) {
        var children = Array.from(node.children);
        console.assert(children.length >= 1);
        var deriveOrientation = function () {
            if (node.dataset.layout.startsWith('v') || (node.dataset.orientation && node.dataset.orientation.startsWith('v'))) {
                return EOrientation.VERTICAL;
            }
            return EOrientation.HORIZONTAL;
        };
        var options = Object.assign(ALayoutContainer.deriveOptions(node), {
            orientation: deriveOrientation(),
            stackLayout: node.dataset.mode === 'stacked' || node.dataset.layout.endsWith('stack')
        });
        var r = new LineUpLayoutContainer(node.ownerDocument, options);
        children.forEach(function (c) { return r.push(derive(c)); });
        return r;
    };
    LineUpLayoutContainer.prototype.defaultOptions = function () {
        return Object.assign(_super.prototype.defaultOptions.call(this), {
            stackLayout: false
        });
    };
    return LineUpLayoutContainer;
}(ASequentialLayoutContainer));
export default LineUpLayoutContainer;
//# sourceMappingURL=LineUpLayoutContainer.js.map