/**
 * Created by Samuel Gratzl on 01.08.2017.
 */
import { ALayoutContainer } from './ALayoutContainer';
import TabbingLayoutContainer from './TabbingLayoutContainer';
import SplitLayoutContainer from './SplitLayoutContainer';
import { dropAble } from 'phovea_core/src/internal/dnd';
import { EOrientation } from './interfaces';
import { AParentLayoutContainer } from './AParentLayoutContainer';
import LineUpLayoutContainer from './LineUpLayoutContainer';
function determineDropArea(x, y) {
    if (x > 0.2 && x < 0.4 && y > 0.3 && y < 0.7) {
        return 'horizontal-scroll';
    }
    if (x > 0.6 && x < 0.8 && y > 0.3 && y < 0.7) {
        return 'vertical-scroll';
    }
    if (x < 0.2) {
        return 'left';
    }
    if (x > 0.8) {
        return 'right';
    }
    if (y < 0.3) {
        return 'top';
    }
    if (y > 0.7) {
        return 'bottom';
    }
    return 'center';
}
export function dropViews(node, reference) {
    node.dataset.drop = 'center';
    node.insertAdjacentHTML('beforeend', "\n    <div class=\"phovea-drop-locations-overlay\">\n      <div></div>\n      <div></div>\n      <div></div>\n    </div>\n  ");
    dropAble(node, [ALayoutContainer.MIME_TYPE], function (result, e) {
        var area = determineDropArea(e.offsetX / node.offsetWidth, e.offsetY / node.offsetHeight);
        var id = parseInt(result.data[ALayoutContainer.MIME_TYPE], 10);
        console.assert(reference.parent != null);
        var item = reference.parent.rootParent.find(id);
        if (item === null) {
            return false;
        }
        return dropLogic(item, reference, area);
    }, function (e) {
        node.dataset.drop = determineDropArea(e.offsetX / node.offsetWidth, e.offsetY / node.offsetHeight);
    }, true);
}
function dropLogic(item, reference, area) {
    if (item instanceof AParentLayoutContainer && reference.parents.indexOf(item) >= 0) {
        //can drop item within one of its children
        return false;
    }
    var parent = reference.parent;
    var canDirectly = parent.canDrop(area);
    if (canDirectly) {
        if (parent.children.indexOf(item) < 0 && item !== reference) {
            return parent.place(item, reference, area); //tod
        }
        return false; //already a child
    }
    if (area === 'center' && item !== reference) {
        //replace myself with a tab container
        var p_1 = new TabbingLayoutContainer(item.node.ownerDocument, {});
        parent.replace(reference, p_1);
        p_1.push(reference);
        p_1.push(item);
        p_1.active = item;
        return true;
    }
    //corner case if I'm the child of a tabbing, tab that and not me
    if (parent instanceof TabbingLayoutContainer && !(area === 'horizontal-scroll' || area === 'vertical-scroll')) {
        return dropLogic(item, parent, area);
    }
    if (parent === reference || item === reference) {
        //can't split my parent with my parent
        return false;
    }
    if (area === 'horizontal-scroll' || area === 'vertical-scroll') {
        var orientation_1 = area === 'horizontal-scroll' ? EOrientation.HORIZONTAL : EOrientation.VERTICAL;
        var p_2 = new LineUpLayoutContainer(item.node.ownerDocument, {
            orientation: orientation_1,
            stackLayout: true
        });
        parent.replace(reference, p_2);
        p_2.push(reference);
        p_2.push(item);
        return true;
    }
    //replace myself with a split container
    var p = new SplitLayoutContainer(item.node.ownerDocument, {
        orientation: (area === 'left' || area === 'right') ? EOrientation.HORIZONTAL : EOrientation.VERTICAL,
        name: (area === 'left' || area === 'top') ? item.name + "|" + reference.name : reference.name + "|" + item.name
    });
    parent.replace(reference, p);
    if (area === 'left' || area === 'top') {
        p.push(autoWrap(item), -1, 0.5);
        p.push(reference, -1, 0.5);
    }
    else {
        p.push(reference, -1, 0.5);
        p.push(autoWrap(item), -1, 0.5);
    }
    //force ratios
    p.ratios = [0.5, 0.5];
    return true;
}
function autoWrap(item) {
    if (item.autoWrapOnDrop) {
        return new TabbingLayoutContainer(item.node.ownerDocument, { name: typeof item.autoWrapOnDrop === 'string' ? item.autoWrapOnDrop : 'Side' }, item);
    }
    return item;
}
//# sourceMappingURL=dropper.js.map